/*
 * $Id: smb.h,v 1.5 2017/08/30 21:17:10 cyfm Exp $
 *
 * :ts=8
 *
 * smb.h
 *
 * Copyright (C) 1995 by Paal-Kr. Engstad and Volker Lendecke
 * Modified for use with AmigaOS by Olaf Barthel <obarthel -at- gmx -dot- net>
 */

#ifndef _LINUX_SMB_H
#define _LINUX_SMB_H

#define SMB_PORT 139
#define SMB_MAXNAMELEN 255

#ifdef __MOSDOS64__
#include <inttypes.h>
typedef uint8_t  byte;
typedef uint16_t word;
typedef uint32_t dword;
typedef uint64_t qword;
#else
typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;
#endif

/*
 * Set/Get values in SMB-byte order
 */

#define PVAL(buf,pos)   ((unsigned)BVAL(buf,pos))

#define BVAL(buf,pos)   (((unsigned char *)(buf))[pos])
#define WVAL(buf,pos)   (((word)PVAL(buf,pos))|((word)PVAL(buf,(pos)+1))<<8)
#define DVAL(buf,pos)   (((dword)WVAL(buf,pos))|((dword)WVAL(buf,(pos)+2))<<16)
#ifdef __MOSDOS64__
#define QVAL(buf,pos)   (((qword)DVAL(buf,pos))|((qword)DVAL(buf,(pos)+4)<<32))
#endif

#define BSET(buf,pos,val)     (BVAL(buf,pos)=((val) & 0xFF))
#define WSET(buf,pos,val) do { BVAL(buf,pos)=((val) & 0xFF); BVAL(buf,(pos)+1)=(((unsigned)(val))>>8) & 0xFF; } while (0)
#define DSET(buf,pos,val) do { WSET(buf,pos,(val)); WSET(buf,(pos)+2,((unsigned)(val))>>16); } while (0)
#ifdef __MOSDOS64__
#define QSET(buf,pos,val) do { DSET(buf,pos,(val)); DSET(buf,(pos)+4,((unsigned)(val))>>32); } while (0)
#endif

#define smb_base(buf) ((byte *)(((byte *)(buf))+4))

enum smb_protocol
{
  PROTOCOL_NONE,
  PROTOCOL_CORE,
  PROTOCOL_COREPLUS,
  PROTOCOL_LANMAN1,
  PROTOCOL_LANMAN2,
  PROTOCOL_NT1
};

enum smb_conn_state
{
  CONN_VALID,                   /* everything's fine */
  CONN_INVALID,                 /* Something went wrong, but did not
                                   try to reconnect yet. */
  CONN_RETRIED                  /* Tried a reconnection, but was refused */
};

struct smb_dskattr
{
  word total;
  word allocblocks;
  word blocksize;
  word free;
};

#ifdef __MOSDOS64__
struct smb_dskattr64
{
  qword total;
  qword free;
  dword sectorsperunit;
  dword bytespersector;
};
#endif

/*
 * Contains all relevant data on a SMB networked file.
 */
struct smb_dirent
{
  int opened;                   /* is it open on the fileserver? */
  word fileid;                  /* What id to handle a file with? */
  word attr;                    /* Attribute fields, DOS value */

  time_t atime, mtime,
         wtime, ctime;          /* Times, as seen by the server, normalized */
                                /* to UTC. The ugly conversion happens in */
                                /* proc.c */
#ifdef __MOSDOS64__
  qword size;                   /* File size. */
#else
  unsigned long size;           /* File size. */
#endif
  char *complete_path;          /* Complete path, MS-DOS notation, with '\' */
  size_t complete_path_size;	/* Number of bytes allocated for name */
  int len;                      /* Namelength. */
};

#endif /* _LINUX_SMB_H */
